import { Module } from 'module';
import { createContext, runInContext } from 'vm';
import { transformSync, transform } from 'esbuild';

const checkArg = (arg) => {
    const argType = typeof arg;
    if (argType !== 'string') {
        throw new Error(`Argument must be string, not '${argType}'.`);
    }
};

const requireFromString = (code, globals = {}) => {
    checkArg(code);
    const _module = new Module(String(new Date().valueOf()));
    const context = createContext(Object.assign({ exports: _module.exports, module: _module, require }, globals));
    runInContext(code, context);
    return context.module.exports;
};

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const importFromString = ({ code, transformOptions, globals = {} }) => __awaiter(void 0, void 0, void 0, function* () {
    checkArg(code);
    const { code: transformedCode } = yield transform(code, Object.assign({ format: 'cjs' }, transformOptions));
    return requireFromString(transformedCode, globals);
});
const importFromStringSync = ({ code, transformOptions, globals = {} }) => {
    checkArg(code);
    const { code: transformedCode } = transformSync(code, Object.assign({ format: 'cjs' }, transformOptions));
    return requireFromString(transformedCode, globals);
};

export { importFromString, importFromStringSync, requireFromString };
